unit hiKeyboardParams;

interface

uses Kol,Share,Windows;

type
  THIKeyboardParams = class
   private
   public
    _prop_RepeatDelay:integer;
    _prop_RepeatSpeed:integer;
    _prop_CaretBlinkTime:integer;
    _prop_KeyboardCues:byte;
    _prop_LangToggle:byte;
    _prop_SaveSettings:byte;
    
    _prop_Active:byte;
    _prop_ActivateByNumLock:byte;
    _prop_WarningEnabled:byte;
    
    _prop_Enabled:byte;
    _prop_AudibleFeedback:byte;
    _prop_ActivateByShift:byte;
    _prop_SoundByShift:byte;
    _prop_TriState:byte;
    _prop_TwoKeysOff:byte;
    _prop_Indicator:byte;
    

    _data_RepeatDelay:THI_Event;
    _data_RepeatSpeed:THI_Event;
    _data_CaretBlinkTime:THI_Event;
    _data_KeyboardCues:THI_Event;
    _data_LangToggle:THI_Event;
    _data_SaveSettings:THI_Event;

    _data_ActivateToggleSound:THI_Event;
    _data_ActivateByNumLock:THI_Event;
    _data_WarningEnabled:THI_Event;

    _data_StickyKeysEnabled:THI_Event;
    _data_AudibleFeedback:THI_Event;
    _data_ActivateByShift:THI_Event;
    _data_SoundByShift:THI_Event;
    _data_TriState:THI_Event;
    _data_TwoKeysOff:THI_Event;
    _data_Indicator:THI_Event;
    
    
    procedure _work_doBlockInput(var _Data:TData; Index:word);   
    procedure _work_doSetRepeatDelay(var _Data:TData; Index:word);
    procedure _work_doSetRepeatSpeed(var _Data:TData; Index:word);
    procedure _work_doSetLangToggle(var _Data:TData; Index:word);
    procedure _work_doSetCaretBlinkTime(var _Data:TData; Index:word);
    procedure _work_doSetKeyboardCues(var _Data:TData; Index:word);
    
    procedure _work_doSetToggleSound(var _Data:TData; Index:word);
    procedure _work_doSetStickyKeys(var _Data:TData; Index:word);

    procedure _var_CurRepeatDelay(var _Data:TData; Index:word);
    procedure _var_CurRepeatSpeed(var _Data:TData; Index:word);
    procedure _var_CurCaretBlinkTime(var _Data:TData; Index:word);
    procedure _var_CurKeyboardCues(var _Data:TData; Index:word);
    procedure _var_CurLangToggle(var _Data:TData; Index:word);
    procedure _var_KeyBoardType(var _Data:TData; Index:word);
    procedure _var_KeyBoardSubType(var _Data:TData; Index:word);
    procedure _var_FuncKeyCount(var _Data:TData; Index:word);
    
    procedure _var_IsToggleSoundActive(var _Data:TData; Index:word);
    procedure _var_IsActivateByNumLock(var _Data:TData; Index:word);
    procedure _var_IsWarningEnabled(var _Data:TData; Index:word);
    
    procedure _var_IsStickyKeysEnabled(var _Data:TData; Index:word);
    procedure _var_IsAudibleFeedback(var _Data:TData; Index:word);
    procedure _var_IsActivateByShift(var _Data:TData; Index:word);
    procedure _var_IsSoundByShift(var _Data:TData; Index:word);
    procedure _var_IsTriState(var _Data:TData; Index:word);
    procedure _var_IsTwoKeysOff(var _Data:TData; Index:word);
    procedure _var_IsIndicatorEnabled(var _Data:TData; Index:word);
   
  end;

const SPI_GETKEYBOARDCUES = $100A; SPI_SETKEYBOARDCUES = $100B;

function BlockInput(fBlockIt:bool):bool; stdcall; external 'user32.dll';

implementation


procedure THIKeyboardParams._work_doBlockInput(var _Data:TData; Index:word);
begin
  BlockInput(bool(ToInteger(_Data)));
end;


procedure THIKeyboardParams._work_doSetRepeatDelay(var _Data:TData; Index:word);
var iDelay, ss:integer;
begin
  iDelay := ReadInteger(_Data, _data_RepeatDelay, _prop_RepeatDelay);
  if (iDelay>-1) and (iDelay<4) then
  begin
    ss := ReadInteger(_Data, _data_SaveSettings, _prop_SaveSettings);
    if ss <> 0 then ss:= SPIF_UPDATEINIFILE or SPIF_SENDCHANGE;
    SystemParametersInfo(SPI_SETKEYBOARDDELAY, iDelay, nil, ss);  
  end;
end;


procedure THIKeyboardParams._work_doSetRepeatSpeed(var _Data:TData; Index:word);
var iSpeed, ss:integer;
begin
  iSpeed := ReadInteger(_Data, _data_RepeatSpeed, _prop_RepeatSpeed);
  if (iSpeed>-1) and (iSpeed<32) then
  begin
    ss := ReadInteger(_Data, _data_SaveSettings, _prop_SaveSettings);
    if ss <> 0 then ss:= SPIF_UPDATEINIFILE or SPIF_SENDCHANGE;
    SystemParametersInfo(SPI_SETKEYBOARDSPEED, iSpeed, nil, ss);  
  end;
end;


procedure THIKeyboardParams._work_doSetLangToggle(var _Data:TData; Index:word);
var iLngToggle, tmp:integer; hkKey:HKey; OldLayHK:string;
begin
  iLngToggle := ReadInteger(_Data, _data_LangToggle, _prop_LangToggle) + 1; 
  if (iLngToggle>0) and (iLngToggle<4) then
  begin

    hkKey := RegKeyOpenWrite(HKEY_CURRENT_USER, 'keyboard layout\toggle');
    OldLayHK := RegKeyGetStr(hkKey, 'Layout Hotkey');

    if iLngToggle<>3 then
    begin
      if iLngToggle=1 then tmp:=2 else tmp:=1;
      if RegKeyValExists(hkKey, 'Language Hotkey') then RegKeySetStr(hkKey, 'Language Hotkey', Int2Str(iLngToggle));
      RegKeySetStr(hkKey, 'Hotkey', Int2Str(iLngToggle));
      if (OldLayHK<>'') and (OldLayHK<>'3') then RegKeySetStr(hkKey, 'Layout Hotkey', Int2Str(tmp));     
    end
    else
    begin
      if RegKeyValExists(hkKey, 'Language Hotkey') then RegKeySetStr(hkKey, 'Language Hotkey', '3');
      RegKeySetStr(hkKey, 'Hotkey', '3');    
    end;
    SystemParametersInfo(SPI_SETLANGTOGGLE, 0, nil, 0); //     
    RegKeyClose(hkKey);      
  end; 
  
end;


procedure THIKeyboardParams._work_doSetCaretBlinkTime(var _Data:TData; Index:word);
var bt:integer; hkKey:HKey;
begin
  bt:=ReadInteger(_Data, _data_CaretBlinkTime, _prop_CaretBlinkTime);
  SetCaretBlinkTime(bt);
  if ReadInteger(_Data, _data_SaveSettings, _prop_SaveSettings) <> 0 then
  begin
    hkKey := RegKeyOpenWrite(HKEY_CURRENT_USER, 'Control Panel\Desktop');
    RegKeySetStr(hkKey, 'CursorBlinkRate', Int2Str(bt));
    RegKeyClose(hkKey); 
  end;
end;


procedure THIKeyboardParams._work_doSetKeyboardCues(var _Data:TData; Index:word);
var iCues, ss:integer;
begin
  iCues := ReadInteger(_Data, _data_KeyboardCues, _prop_KeyboardCues);
  ss := ReadInteger(_Data, _data_SaveSettings, _prop_SaveSettings);
  if ss <> 0 then ss:= SPIF_UPDATEINIFILE or SPIF_SENDCHANGE;
  SystemParametersInfo(SPI_SETKEYBOARDCUES, 0, @iCues, ss);
end;


procedure THIKeyboardParams._work_doSetToggleSound(var _Data:TData; Index:word);
var iFlags, ss:integer; tkTogKeys:TToggleKeys;
begin
  tkTogKeys.cbSize:=SizeOf(TToggleKeys);
  iFlags:=TKF_INDICATOR or TKF_CONFIRMHOTKEY;
  if ReadInteger(_Data, _data_ActivateToggleSound, _prop_Active) = 1 then iFlags:=iFlags or TKF_TOGGLEKEYSON;
  if ReadInteger(_Data, _data_ActivateByNumLock, _prop_ActivateByNumLock) = 1 then iFlags:=iFlags or TKF_HOTKEYACTIVE;
  if ReadInteger(_Data, _data_WarningEnabled, _prop_WarningEnabled) = 1 then iFlags:=iFlags or TKF_HOTKEYSOUND;
  tkTogKeys.dwFlags:=iFlags;                  
  ss := ReadInteger(_Data, _data_SaveSettings, _prop_SaveSettings);
  if ss <> 0 then ss:= SPIF_UPDATEINIFILE or SPIF_SENDCHANGE;
  SystemParametersInfo(SPI_SETTOGGLEKEYS, SizeOf(TToggleKeys), @tkTogKeys, ss);  
end;


procedure THIKeyboardParams._work_doSetStickyKeys(var _Data:TData; Index:word);
var iFlags, ss:integer; skStickKeys:TStickyKeys;
begin
  skStickKeys.cbSize:=SizeOf(TStickyKeys);
  iFlags:=SKF_CONFIRMHOTKEY;
  
  if ReadInteger(_Data, _data_StickyKeysEnabled, _prop_Enabled) = 1 then iFlags:=iFlags or SKF_STICKYKEYSON;
  if ReadInteger(_Data, _data_AudibleFeedback, _prop_AudibleFeedback) = 1 then iFlags:=iFlags or SKF_AUDIBLEFEEDBACK;
  if ReadInteger(_Data, _data_ActivateByShift, _prop_ActivateByShift) = 1 then iFlags:=iFlags or SKF_HOTKEYACTIVE;
  if ReadInteger(_Data, _data_SoundByShift, _prop_SoundByShift) = 1 then iFlags:=iFlags or SKF_HOTKEYSOUND;
  if ReadInteger(_Data, _data_TriState, _prop_TriState) = 1 then iFlags:=iFlags or SKF_TRISTATE;
  if ReadInteger(_Data, _data_TwoKeysOff, _prop_TwoKeysOff) = 1 then iFlags:=iFlags or SKF_TWOKEYSOFF;
  if ReadInteger(_Data, _data_Indicator, _prop_Indicator) = 1 then iFlags:=iFlags or SKF_INDICATOR;
  
  skStickKeys.dwFlags:=iFlags;                  
  ss := ReadInteger(_Data, _data_SaveSettings, _prop_SaveSettings);
  if ss <> 0 then ss:= SPIF_UPDATEINIFILE or SPIF_SENDCHANGE;
  SystemParametersInfo(SPI_SETSTICKYKEYS, SizeOf(TStickyKeys), @skStickKeys, ss);  
end;


procedure THIKeyboardParams._var_CurRepeatDelay;
var iDelay:integer;
begin
  SystemParametersInfo(SPI_GETKEYBOARDDELAY, 0, @iDelay, 0);
  dtInteger(_Data, iDelay);
end;


procedure THIKeyboardParams._var_CurRepeatSpeed;
var iSpeed:DWORD;
begin
  SystemParametersInfo(SPI_GETKEYBOARDSPEED, 0, @iSpeed, 0);
  dtInteger(_Data, iSpeed);
end;


procedure THIKeyboardParams._var_CurCaretBlinkTime;
begin
  dtInteger(_Data, GetCaretBlinkTime);
end;


procedure THIKeyboardParams._var_CurKeyboardCues;
var bCues:bool;
begin
  SystemParametersInfo(SPI_GETKEYBOARDCUES, 0, @bCues, 0);
  dtInteger(_Data, Integer(bCues));
end;


procedure THIKeyboardParams._var_CurLangToggle;
var hkKey:HKey;
begin
  hkKey := RegKeyOpenRead(HKEY_CURRENT_USER, 'keyboard layout\toggle');
  dtInteger(_Data, str2int(RegKeyGetStr(hkKey, 'Hotkey'))-1);
  RegKeyClose(hkKey); 
end;


procedure THIKeyboardParams._var_IsToggleSoundActive(var _Data:TData; Index:word);
var iActive:integer; tkTogKeys:TToggleKeys;
begin
  tkTogKeys.cbSize:=SizeOf(TToggleKeys);
  SystemParametersInfo(SPI_GETTOGGLEKEYS, SizeOf(TToggleKeys), @tkTogKeys, 0);
  if (tkTogKeys.dwFlags and TKF_TOGGLEKEYSON)<> 0 then iActive:=1 else iActive:=0;
  dtInteger(_Data, iActive);    
end;


procedure THIKeyboardParams._var_IsActivateByNumLock(var _Data:TData; Index:word);
var iActive:integer; tkTogKeys:TToggleKeys;
begin
  tkTogKeys.cbSize:=SizeOf(TToggleKeys);
  SystemParametersInfo(SPI_GETTOGGLEKEYS, SizeOf(TToggleKeys), @tkTogKeys, 0);
  if (tkTogKeys.dwFlags and TKF_HOTKEYACTIVE)<> 0 then iActive:=1 else iActive:=0;
  dtInteger(_Data, iActive);    
end;
 
           
procedure THIKeyboardParams._var_IsWarningEnabled(var _Data:TData; Index:word);
var iEnabled:integer; tkTogKeys:TToggleKeys;
begin
  tkTogKeys.cbSize:=SizeOf(TToggleKeys);
  SystemParametersInfo(SPI_GETTOGGLEKEYS, SizeOf(TToggleKeys), @tkTogKeys, 0);
  if (tkTogKeys.dwFlags and TKF_HOTKEYSOUND)<> 0 then iEnabled:=1 else iEnabled:=0;
  dtInteger(_Data, iEnabled);    
end;
    

procedure THIKeyboardParams._var_IsStickyKeysEnabled(var _Data:TData; Index:word);
var iEnabled:integer; skStickKeys:TStickyKeys;
begin
  skStickKeys.cbSize:=SizeOf(TStickyKeys);
  SystemParametersInfo(SPI_GETSTICKYKEYS, SizeOf(TStickyKeys), @skStickKeys, 0);
  if (skStickKeys.dwFlags and SKF_STICKYKEYSON)<> 0 then iEnabled:=1 else iEnabled:=0;
  dtInteger(_Data, iEnabled);
end;


procedure THIKeyboardParams._var_IsAudibleFeedback(var _Data:TData; Index:word);
var iEnabled:integer; skStickKeys:TStickyKeys;
begin
  skStickKeys.cbSize:=SizeOf(TStickyKeys);
  SystemParametersInfo(SPI_GETSTICKYKEYS, SizeOf(TStickyKeys), @skStickKeys, 0);
  if (skStickKeys.dwFlags and SKF_AUDIBLEFEEDBACK)<> 0 then iEnabled:=1 else iEnabled:=0;
  dtInteger(_Data, iEnabled);
end;


procedure THIKeyboardParams._var_IsActivateByShift(var _Data:TData; Index:word);
var iEnabled:integer; skStickKeys:TStickyKeys;
begin
  skStickKeys.cbSize:=SizeOf(TStickyKeys);
  SystemParametersInfo(SPI_GETSTICKYKEYS, SizeOf(TStickyKeys), @skStickKeys, 0);
  if (skStickKeys.dwFlags and SKF_HOTKEYACTIVE)<> 0 then iEnabled:=1 else iEnabled:=0;
  dtInteger(_Data, iEnabled);
end;


procedure THIKeyboardParams._var_IsSoundByShift(var _Data:TData; Index:word);
var iSound:integer; skStickKeys:TStickyKeys;
begin
  skStickKeys.cbSize:=SizeOf(TStickyKeys);
  SystemParametersInfo(SPI_GETSTICKYKEYS, SizeOf(TStickyKeys), @skStickKeys, 0);
  if (skStickKeys.dwFlags and SKF_HOTKEYSOUND)<> 0 then iSound:=1 else iSound:=0;
  dtInteger(_Data, iSound);
end;


procedure THIKeyboardParams._var_IsTriState(var _Data:TData; Index:word);
var iState:integer; skStickKeys:TStickyKeys;
begin
  skStickKeys.cbSize:=SizeOf(TStickyKeys);
  SystemParametersInfo(SPI_GETSTICKYKEYS, SizeOf(TStickyKeys), @skStickKeys, 0);
  if (skStickKeys.dwFlags and SKF_TRISTATE)<> 0 then iState:=1 else iState:=0;
  dtInteger(_Data, iState);
end;


procedure THIKeyboardParams._var_IsTwoKeysOff(var _Data:TData; Index:word);
var iState:integer; skStickKeys:TStickyKeys;
begin
  skStickKeys.cbSize:=SizeOf(TStickyKeys);
  SystemParametersInfo(SPI_GETSTICKYKEYS, SizeOf(TStickyKeys), @skStickKeys, 0);
  if (skStickKeys.dwFlags and SKF_TWOKEYSOFF)<> 0 then iState:=1 else iState:=0;
  dtInteger(_Data, iState);
end;


procedure THIKeyboardParams._var_IsIndicatorEnabled(var _Data:TData; Index:word);
var iState:integer; skStickKeys:TStickyKeys;
begin
  skStickKeys.cbSize:=SizeOf(TStickyKeys);
  SystemParametersInfo(SPI_GETSTICKYKEYS, SizeOf(TStickyKeys), @skStickKeys, 0);
  if (skStickKeys.dwFlags and SKF_INDICATOR)<> 0 then iState:=1 else iState:=0;
  dtInteger(_Data, iState);
end;


procedure THIKeyboardParams._var_KeyBoardType(var _Data:TData; Index:word);
begin
  dtInteger(_Data, GetKeyboardType(0));
end;


procedure THIKeyboardParams._var_KeyBoardSubType(var _Data:TData; Index:word);
begin
  dtInteger(_Data, GetKeyboardType(1));
end;


procedure THIKeyboardParams._var_FuncKeyCount(var _Data:TData; Index:word);
begin
  dtInteger(_Data, GetKeyboardType(2));
end;

end.
